<?php
/**
 * The Juno_Books class provides the functions necessary for the book object
 * in Juno and is stored in the Horde_DataTree backend.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: juno/lib/Books.php,v 1.23 2005/01/03 14:35:10 jan Exp $
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @package Juno
 */
class Juno_Books {

    var $_books;

    function Juno_Books()
    {
        global $conf;

        if (empty($conf['datatree']['driver'])) {
            Horde::fatal(_("You must configure a Horde DataTree backend to use Juno."), __FILE__, __LINE__);
        }
        $driver = $conf['datatree']['driver'];
        $params = array_merge(Horde::getDriverConfig('datatree', $driver), array('group' => 'juno.books'));

        $this->_books = &DataTree::singleton($driver, $params);
    }

    function &singleton()
    {
        static $book;

        if (!isset($book)) {
            $book = new Juno_Books();
        }

        return $book;
    }

    function &newBook($name)
    {
        if (empty($name)) {
            return PEAR::raiseError(_("Can not create a book with an empty name."));
        }
        $book = &new Book_DataTreeObject($name);
        $book->setBooksOb($this);
        return $book;
    }

    function &getBook($book_id)
    {
        $book = &$this->_books->getObjectById($book_id, 'Book_DataTreeObject');
        $book->setBooksOb($this);
        return $book;
    }

    function getBookId($book_name)
    {
        return $this->_books->getId($book_name);
    }

    function getBookParent($book_name)
    {
        return $this->_books->getParent($book_name);
    }

    function getBookPath($book_id)
    {
        return $this->_books->getName($book_id);
    }

    /**
     * Save book to DataTree backend.
     */
    function saveBook(&$info)
    {
        if (empty($info['id'])) {
            $book_path = ($info['parent_id'] != DATATREE_ROOT) ? $this->getBookPath($info['parent_id']) . ':' : '';
            $book_path .= $info['name'];
            $book = &$this->newBook($book_path);
            $book->data['message_seq'] = 0;
        } else {
            $book = $this->getBook($info['id']);
        }

        $book->data['start_date'] = $info['start_date'];
        $book->data['currency'] = $info['currency'];
        $book->data['multicurrency'] = $info['multicurrency'];

        if (empty($info['id'])) {
            /* New book, add the category object. */
            $result = $this->_books->add($book);
            $info['id'] = $this->_books->getId($book);
        } else {
            /* Existing book, first save the category data. */
            $this->_books->updateData($book);
            $old_parent_id = $this->getBookParent($book);
            $old_book_cat = $book->getName();
            /* Check if the old parent id is still the same. */
            if ($old_parent_id != $info['parent_id']) {
                /* Get the new parent path. */
                $new_parent = $this->_books->getName($info['parent_id']);
                /* Move the book. */
                $move = $this->_books->move($old_book_cat, $new_parent);
                if (is_a($move, 'PEAR_Error')) {
                    return $move;
                }
                $old_book_cat = $new_parent . ':' . $book->getShortName();
            }
            /* If the book name has changed, rename the category. */
            if ($book->getShortName() != $info['name']) {
                $rename = $this->_books->rename($old_book_cat, $info['name']);
                if (is_a($rename, 'PEAR_Error')) {
                    return $rename;
                }
            }
        }

        return $info['id'];
    }

    function getCurrentBookId()
    {
        if (Util::getFormData('close')) {
            unset($_SESSION['juno']['book_id']);
            return null;
        }
        if (($book_id = Util::getFormData('book_id')) !== null) {
            $_SESSION['juno']['book_id'] = $book_id;
        }
        return isset($_SESSION['juno']['book_id']) ?
                    $_SESSION['juno']['book_id'] : null;
    }

    function &getCurrentBook()
    {
        $book_id = $this->getCurrentBookId();
        if (empty($book_id)) {
            return false;
        }

        $book = $this->getBook($book_id);
        return $book;
    }

    function getBooks($root_book = DATATREE_ROOT, $formatted = true, $sort_by = 'name', $sort_dir = 0, $no_root = false)
    {
        /* Get root book name and calculate base indent. */
        $root_book_name = $this->_books->getName($root_book);
        $base_indent = substr_count($root_book_name, ':');

        $books_tree = $this->_books->get(DATATREE_FORMAT_FLAT, $root_book_name, true, _("-- Top Level --"));

        /* If root is not required, unset it out of the array. */
        if ($no_root) {
            unset($books_tree[$root_book]);
            /* If there is an actual base indent, shorten it by one. */
            if ($base_indent > 0) {
                $base_indent--;
            }
        }

        if (!$formatted) {
            return $books_tree;
        }

        $books_list = array();
        $i = 0;
        foreach ($books_tree as $book_id => $book_name) {
            $books_list[$book_id]['class'] = 'item' . ($i++ % 2);

            /* Calculate indent level. */
            $books_list[$book_id]['indentn'] = substr_count($book_name, ':') - $base_indent;
            $books_list[$book_id]['indent'] = str_repeat('.. ', $books_list[$book_id]['indentn']);

            $url = Util::addParameter(Horde::applicationUrl('accounts/'), 'book_id', $book_id);
            $books_list[$book_id]['url'] = $url;
            $books_list[$book_id]['name'] = DataTree::getShortName($book_name);
            /* Action links. */
            $books_list[$book_id]['actions'] = array();

            /* Edit book button. */
            $url = Util::addParameter(Horde::applicationUrl('books/edit.php'), 'id', $book_id);
            $books_list[$book_id]['actions'][] = Horde::link($url, _("Edit book"), '', '', '', _("Edit book")) . Horde::img('edit.png', _("Edit book"), '', $GLOBALS['registry']->getImageDir('horde')) . '</a>';

            /* Delete book button. */
            $url = Util::addParameter(Horde::applicationUrl('books/delete.php'), 'id', $book_id);
            $books_list[$book_id]['actions'][] = Horde::link($url, _("Delete book"), '', '', '', _("Delete book")) . Horde::img('delete-small.png', _("Delete book"), null, $GLOBALS['registry']->getImageDir('horde')) . '</a>';

            /* Open book button. */
            $url = Util::addParameter(Horde::applicationUrl('accounts/index.php'), 'book_id', $book_id);
            $books_list[$book_id]['actions'][] = Horde::link($url, _("Open book"), '', '', '', _("Open book")) . Horde::img('book.png', _("Open book")) . '</a>';
        }

        require_once 'Horde/Array.php';
        Horde_Array::arraySort($books_list, $sort_by, $sort_dir);
        return $books_list;
    }

}

/**
 * Extension of the DataTreeObject class for storing Book
 * information in the DataTree driver.
 *
 * $Horde: juno/lib/Books.php,v 1.23 2005/01/03 14:35:10 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @since   Juno 0.1
 * @package Juno
 */
class Book_DataTreeObject extends DataTreeObject {

    var $_booksOb;

    /**
     * The Book_DataTreeObject constructor.
     *
     * @param string $name The Book to use.
     */
    function Book_DataTreeObject($name)
    {
        global $conf;

        $driver = $conf['datatree']['driver'];
        $driver_conf = array_merge(Horde::getDriverConfig('datatree', $driver),
                                   array('group' => 'juno.books'));
        $this->books = &DataTree::singleton($driver, $driver_conf);

        parent::DataTreeObject($name);
    }

    /**
     * Associates a books object with this page.
     */
    function setBooksOb(&$booksOb)
    {
        $this->_booksOb = &$booksOb;
    }

    function getId()
    {
        return $this->_booksOb->getBookId($this->getName());
    }

    function getBookArray()
    {
        $array['id'] = $this->_booksOb->getBookId($this->getName());
        $array['name'] = $this->getShortName();
        $array['start_date'] = $this->data['start_date'];
        $array['currency'] = $this->data['currency'];
        $array['parent_id'] = $this->_booksOb->getBookParent($this);
        $array['multicurrency'] = $this->data['multicurrency'];
        return $array;
    }

    function has($check)
    {
        switch ($check) {
        case 'multicurrency':
            return $this->data['multicurrency'];
        }
    }

    function getCurrency()
    {
        return $this->data['currency'];
    }

    function save()
    {
        return $this->_booksOb->storePage($this);
    }

    function _toAttributes()
    {
        $attributes[] = array('name'  => 'book',
                              'key'   => 'start_date',
                              'value' => $this->data['start_date']);
        $attributes[] = array('name'  => 'book',
                              'key'   => 'currency',
                              'value' => $this->data['currency']);
        $attributes[] = array('name'  => 'book',
                              'key'   => 'multicurrency',
                              'value' => $this->data['multicurrency']);
        return $attributes;
    }

    function _fromAttributes($attributes)
    {
        /* Initialise the data array. */
        $this->data = array();

        foreach ($attributes as $attr) {
            $this->data[$attr['key']] = $attr['value'];
        }
    }

}
